@@ -23,7 +23,7 @@
 import numpy as np
 from tensorflow.keras.callbacks import ModelCheckpoint
 import os
-import csv
+import json
 # Internal libraries/scripts
 from miscnn.data_loading.data_io import create_directories, backup_history
 from miscnn.utils.plotting import plot_validation
@@ -49,15 +49,15 @@ Args:
     run_detailed_evaluation (boolean):      Option if a detailed evaluation (additional prediction) should be performed.
     callbacks (list of Callback classes):   A list of Callback classes for custom evaluation.
     save_models (boolean):                  Option if fitted models should be stored or thrown away.
-    direct_output (boolean):                Option, if computed evaluations will be output as the return of this function or
+    return_output (boolean):                Option, if computed evaluations will be output as the return of this function or
                                             if the evaluations will be saved on disk in the evaluation directory.
 """
 def cross_validation(sample_list, model, k_fold=3, epochs=20,
                      iterations=None, evaluation_path="evaluation",
-                     draw_figures=True, run_detailed_evaluation=True,
-                     callbacks=[], save_models=True, direct_output=False):
+                     draw_figures=False, run_detailed_evaluation=False,
+                     callbacks=[], save_models=True, return_output=False):
     # Initialize result cache
-    if direct_output : validation_results = []
+    if return_output : validation_results = []
     # Randomly permute the sample list
     samples_permuted = np.random.permutation(sample_list)
     # Split sample list into folds
@@ -83,7 +83,7 @@ def cross_validation(sample_list, model, k_fold=3, epochs=20,
         history = model.evaluate(training, validation, epochs=epochs,
                                  iterations=iterations, callbacks=cb_list)
         # Backup current history dictionary
-        if direct_output : validation_results.append(history.history)
+        if return_output : validation_results.append(history.history)
         else : backup_history(history.history, subdir)
         # Draw plots for the training & validation
         if draw_figures:
@@ -92,7 +92,7 @@ def cross_validation(sample_list, model, k_fold=3, epochs=20,
         if run_detailed_evaluation:
             detailed_validation(validation, model, subdir)
     # Return the validation results
-    if direct_output : return validation_results
+    if return_output : return validation_results
 
 
 #-----------------------------------------------------#
@@ -122,9 +122,9 @@ def split_folds(sample_list, k_fold=3, evaluation_path="evaluation"):
         validation = folds[i]
         # Initialize evaluation subdirectory for current fold
         subdir = create_directories(evaluation_path, "fold_" + str(i))
-        fold_cache = os.path.join(subdir, "sample_list.csv")
+        fold_cache = os.path.join(subdir, "sample_list.json")
         # Write sampling to disk
-        write_fold2csv(fold_cache, training, validation)
+        write_fold2disk(fold_cache, training, validation)
 
 
 
@@ -149,8 +149,8 @@ def run_fold(fold, model, epochs=20, iterations=None,
              save_models=True):
     # Load sampling fold from disk
     fold_path = os.path.join(evaluation_path, "fold_" + str(fold),
-                             "sample_list.csv")
-    training, validation = load_csv2fold(fold_path)
+                             "sample_list.json")
+    training, validation = load_disk2fold(fold_path)
     # Reset Neural Network model weights
     model.reset_weights()
     # Initialize evaluation subdirectory for current fold
@@ -171,22 +171,21 @@ def run_fold(fold, model, epochs=20, iterations=None,
         plot_validation(history.history, model.metrics, subdir)
 
 #-----------------------------------------------------#
-#                   CSV Management                    #
+#                   JSON Management                   #
 #-----------------------------------------------------#
 # Subfunction for writing a fold sampling to disk
-def write_fold2csv(file_path, training, validation):
-    with open(file_path, "w") as csvfile:
-        writer = csv.writer(csvfile, delimiter=" ")
-        writer.writerow(["TRAINING:"] + list(training))
-        writer.writerow(["VALIDATION:"] + list(validation))
+def write_fold2disk(file_path, training, validation):
+    with open(file_path, "w") as jsonfile:
+        sampling = {"TRAINING": list(training),
+                    "VALIDATION": list(validation)}
+        json.dump(sampling, jsonfile)
 
 # Subfunction for loading a fold sampling from disk
-def load_csv2fold(file_path):
-    training = None
-    validation = None
-    with open(file_path, "r") as csvfile:
-        reader = csv.reader(csvfile, delimiter=" ")
-        for row in reader:
-            if not training : training = row[1:]
-            else : validation = row[1:]
+def load_disk2fold(file_path):
+    with open(file_path, "r") as jsonfile:
+        sampling = json.load(jsonfile)
+        if "TRAINING" in sampling : training = sampling["TRAINING"]
+        else : training = None
+        if "VALIDATION" in sampling : validation = sampling["VALIDATION"]
+        else : validation = None
     return training, validation
