@@ -3,6 +3,9 @@
 #  Copyright:    2020 IT-Infrastructure for Translational Medical Research,    #
 #                University of Augsburg                                        #
 #                                                                              #
+#  Contributions: Michael Lempart, 2020 Department of Radiation Physics,       #
+#                                  Lund University                             #
+#                                                                              #
 #  This program is free software: you can redistribute it and/or modify        #
 #  it under the terms of the GNU General Public License as published by        #
 #  the Free Software Foundation, either version 3 of the License, or           #
@@ -23,6 +26,8 @@
 import numpy as np
 from tensorflow.keras.utils import to_categorical
 import threading
+import multiprocessing as mp
+from functools import partial
 # Internal libraries/scripts
 from miscnn.processing.data_augmentation import Data_Augmentation
 from miscnn.processing.batch_creation import create_batches
@@ -65,11 +70,12 @@ class Preprocessor:
                                                 For Example: (64,128,128) for 64x128x128 patch cubes.
                                                 Be aware that the x-axis represents the number of slices in 3D volumes.
                                                 This parameter will be redundant if fullimage or patchwise-crop analysis is selected!!
+        use_multiprocessing (boolean):          Uses multi-threading to prepare subfunctions if True (parallelized).
     """
     def __init__(self, data_io, batch_size, subfunctions=[],
                  data_aug=Data_Augmentation(), prepare_subfunctions=False,
                  prepare_batches=False, analysis="patchwise-crop",
-                 patch_shape=None):
+                 patch_shape=None, use_multiprocessing=False):
         # Parse Data Augmentation
         if isinstance(data_aug, Data_Augmentation):
             self.data_augmentation = data_aug
@@ -92,6 +98,7 @@ class Preprocessor:
         self.prepare_batches = prepare_batches
         self.analysis = analysis
         self.patch_shape = patch_shape
+        self.use_multiprocessing = use_multiprocessing
 
     #---------------------------------------------#
     #               Class variables               #
@@ -105,6 +112,7 @@ class Preprocessor:
                                             # The function create_batches will use this queue to create batches
     cache = dict()                          # Cache additional information and data for patch assembling after patchwise prediction
     thread_lock = threading.Lock()          # Create a threading lock for multiprocessing
+    mp_threads = 5                          # Number of threads used to prepare subfunctions if use_multiprocessing is set to True
 
     #---------------------------------------------#
     #               Prepare Batches               #
@@ -208,17 +216,29 @@ class Preprocessor:
     #---------------------------------------------#
     #               Run Subfunctions              #
     #---------------------------------------------#
-    # Preprocess data through subfunctions and backup samples
+    # Iterate over all samples, process subfunctions on each and backup
+    # preprocessed samples to disk
     def run_subfunctions(self, indices_list, training=True):
-        # Iterate over all samples
-        for index in indices_list:
-            # Load sample
-            sample = self.data_io.sample_loader(index, load_seg=training)
-            # Run provided subfunctions on imaging data
-            for sf in self.subfunctions:
-                sf.preprocessing(sample, training=training)
-            # Backup sample as pickle to disk
-            self.data_io.backup_sample(sample)
+        # Prepare subfunctions using single threading
+        if not self.use_multiprocessing:
+            for index in indices_list:
+                self.prepare_sample_subfunctions(index, training)
+        # Prepare subfunctions using multiprocessing
+        else:
+            pool = mp.Pool(int(self.mp_threads))
+            pool.map(partial(self.prepare_sample_subfunctions,
+                             training=training),
+                     indices_list)
+
+    # Wrapper function to process subfunctions for a single sample
+    def prepare_sample_subfunctions(self, index, training):
+        # Load sample
+        sample = self.data_io.sample_loader(index, load_seg=training)
+        # Run provided subfunctions on imaging data
+        for sf in self.subfunctions:
+            sf.preprocessing(sample, training=training)
+        # Backup sample as pickle to disk
+        self.data_io.backup_sample(sample)
 
     #---------------------------------------------#
     #           Patch-wise grid Analysis          #
